module type S = sig
type t = string
(** Alias for {!String.t} *)

val empty : string
(** @since 4.13.0: val empty : string *)

(*
val of_bytes : bytes -> string
*)

val of_bytes : Stdcompat__init.bytes -> string

(** @since 4.13.0: val of_bytes : bytes -> string
 *)

(*
val to_bytes : string -> bytes
*)

val to_bytes : string -> Stdcompat__init.bytes

(** @since 4.13.0: val to_bytes : string -> bytes
 *)

val cat : string -> string -> string
(** @since 4.13.0: val cat : string -> string -> string *)

val starts_with : prefix:string -> string -> bool
(** @since 4.13.0: val starts_with : prefix:string -> string -> bool *)

val ends_with : suffix:string -> string -> bool
(** @since 4.13.0: val ends_with : suffix:string -> string -> bool *)

val fold_left : ('a -> char -> 'a) -> 'a -> string -> 'a
(** @since 4.13.0: val fold_left : ('a -> char -> 'a) -> 'a -> string -> 'a *)

val fold_right : (char -> 'a -> 'a) -> string -> 'a -> 'a
(** @since 4.13.0: val fold_right : (char -> 'a -> 'a) -> string -> 'a -> 'a *)

val for_all : (char -> bool) -> string -> bool
(** @since 4.13.0: val for_all : (char -> bool) -> string -> bool *)

val exists : (char -> bool) -> string -> bool
(** @since 4.13.0: val exists : (char -> bool) -> string -> bool *)

val get_uint8 : string -> int -> int
(** @since 4.13.0: val get_uint8 : string -> int -> int *)

val get_int8 : string -> int -> int
(** @since 4.13.0: val get_int8 : string -> int -> int *)

val get_uint16_ne : string -> int -> int
(** @since 4.13.0: val get_uint16_ne : string -> int -> int *)

val get_uint16_be : string -> int -> int
(** @since 4.13.0: val get_uint16_be : string -> int -> int *)

val get_uint16_le : string -> int -> int
(** @since 4.13.0: val get_uint16_le : string -> int -> int *)

val get_int16_ne : string -> int -> int
(** @since 4.13.0: val get_int16_ne : string -> int -> int *)

val get_int16_be : string -> int -> int
(** @since 4.13.0: val get_int16_be : string -> int -> int *)

val get_int16_le : string -> int -> int
(** @since 4.13.0: val get_int16_le : string -> int -> int *)

val get_int32_ne : string -> int -> int32
(** @since 4.13.0: val get_int32_ne : string -> int -> int32 *)

val get_int32_be : string -> int -> int32
(** @since 4.13.0: val get_int32_be : string -> int -> int32 *)

val get_int32_le : string -> int -> int32
(** @since 4.13.0: val get_int32_le : string -> int -> int32 *)

val get_int64_ne : string -> int -> int64
(** @since 4.13.0: val get_int64_ne : string -> int -> int64 *)

val get_int64_be : string -> int -> int64
(** @since 4.13.0: val get_int64_be : string -> int -> int64 *)

val get_int64_le : string -> int -> int64
(** @since 4.13.0: val get_int64_le : string -> int -> int64 *)


val to_seq : t -> char Seq.t

(*
val to_seq : t -> char Stdcompat__seq.t
*)
(** @since 4.07.0: val to_seq : t -> char Seq.t
 *)


val to_seqi : t -> (int * char) Seq.t

(*
val to_seqi : t -> (int * char) Stdcompat__seq.t
*)
(** @since 4.07.0: val to_seqi : t -> (int * char) Seq.t
 *)


val of_seq : char Seq.t -> t

(*
val of_seq : char Stdcompat__seq.t -> t
*)
(** @since 4.07.0: val of_seq : char Seq.t -> t
 *)

val index_from_opt : string -> int -> char -> int option
(** @since 4.05.0: val index_from_opt : string -> int -> char -> int option *)

val rindex_from_opt : string -> int -> char -> int option
(** @since 4.05.0: val rindex_from_opt : string -> int -> char -> int option *)

val index_opt : string -> char -> int option
(** @since 4.05.0: val index_opt : string -> char -> int option *)

val rindex_opt : string -> char -> int option
(** @since 4.05.0: val rindex_opt : string -> char -> int option *)

val split_on_char : char -> string -> string list
(** @since 4.04.0: val split_on_char : char -> string -> string list *)

val equal : t -> t -> bool
(** @since 4.03.0: val equal : t -> t -> bool *)

val uppercase_ascii : string -> string
(** @since 4.03.0: val uppercase_ascii : string -> string *)

val lowercase_ascii : string -> string
(** @since 4.03.0: val lowercase_ascii : string -> string *)

val capitalize_ascii : string -> string
(** @since 4.03.0: val capitalize_ascii : string -> string *)

val uncapitalize_ascii : string -> string
(** @since 4.03.0: val uncapitalize_ascii : string -> string *)

val init : int -> (int -> char) -> string
(** @since 4.02.0: val init : int -> (int -> char) -> string *)

val mapi : (int -> char -> char) -> string -> string
(** @since 4.02.0: val mapi : (int -> char -> char) -> string -> string *)


external create : int -> bytes = "caml_create_string"

(*

external create : int -> bytes = "caml_create_string"[@@ocaml.deprecated
                                                       "Use Bytes.create instead."]

(*

external create : int -> Stdcompat__init.bytes = "caml_create_string"

(*
external create : int -> Stdcompat__init.bytes = "create_string"
*)

*)

*)
(** @since 4.08.0: external create : int -> bytes = "caml_create_string"
@since 4.02.0:
external create : int -> bytes = "caml_create_string"[@@ocaml.deprecated
                                                       "Use Bytes.create instead."]
 *)


external set : bytes -> int -> char -> unit = "%string_safe_set"

(*

external set : bytes -> int -> char -> unit = "%string_safe_set"[@@ocaml.deprecated
                                                                  "Use Bytes.set instead."]

(*
external set :
  Stdcompat__init.bytes -> int -> char -> unit = "%string_safe_set"
*)

*)
(** @since 4.08.0:
    external set : bytes -> int -> char -> unit = "%string_safe_set"
@since 4.02.0:
external set : bytes -> int -> char -> unit = "%string_safe_set"[@@ocaml.deprecated
                                                                  "Use Bytes.set instead."]
 *)


val blit : string -> int -> bytes -> int -> int -> unit

(*
val blit : string -> int -> Stdcompat__init.bytes -> int -> int -> unit
*)
(** @since 4.02.0: val blit : string -> int -> bytes -> int -> int -> unit
 *)


val fill : bytes -> int -> int -> char -> unit

(*

val fill : bytes -> int -> int -> char -> unit[@@ocaml.deprecated
                                                "Use Bytes.fill instead."]

(*
val fill : Stdcompat__init.bytes -> int -> int -> char -> unit
*)

*)
(** @since 4.08.0: val fill : bytes -> int -> int -> char -> unit
@since 4.02.0:
val fill : bytes -> int -> int -> char -> unit[@@ocaml.deprecated
                                                "Use Bytes.fill instead."]
 *)


external unsafe_set : bytes -> int -> char -> unit = "%string_unsafe_set"

(*

external unsafe_set : bytes -> int -> char -> unit = "%string_unsafe_set"
[@@ocaml.deprecated "- : bytes -> int -> char -> unit = <fun>"]

(*
external unsafe_set :
  Stdcompat__init.bytes -> int -> char -> unit = "%string_unsafe_set"
*)

*)
(** @since 4.08.0:
    external unsafe_set : bytes -> int -> char -> unit = "%string_unsafe_set"
@since 4.02.0:
external unsafe_set : bytes -> int -> char -> unit = "%string_unsafe_set"
[@@ocaml.deprecated "- : bytes -> int -> char -> unit = <fun>"]
 *)


external unsafe_blit :
  string -> int -> bytes -> int -> int -> unit = "caml_blit_string"[@@noalloc
                                                                    ]

(*

external unsafe_blit :
  string -> int -> bytes -> int -> int -> unit = "caml_blit_string" "noalloc"

(*

external unsafe_blit :
  string -> int -> Stdcompat__init.bytes -> int -> int -> unit =
    "caml_blit_string" "noalloc"

(*
external unsafe_blit :
  string -> int -> Stdcompat__init.bytes -> int -> int -> unit =
    "blit_string" "noalloc"
*)

*)

*)
(** @since 4.03.0:
    external unsafe_blit :
      string -> int -> bytes -> int -> int -> unit = "caml_blit_string"
    [@@noalloc ]
@since 4.02.0:
external unsafe_blit :
  string -> int -> bytes -> int -> int -> unit = "caml_blit_string" "noalloc"
 *)


external unsafe_fill :
  bytes -> int -> int -> char -> unit = "caml_fill_string"[@@noalloc ]

(*

external unsafe_fill :
  bytes -> int -> int -> char -> unit = "caml_fill_string"[@@ocaml.deprecated
                                                            "- : bytes -> int -> int -> char -> unit = <fun>"]
[@@noalloc ]

(*

external unsafe_fill :
  bytes -> int -> int -> char -> unit = "caml_fill_string" "noalloc"[@@ocaml.deprecated
                                                                    "- : bytes -> int -> int -> char -> unit = <fun>"]

(*

external unsafe_fill :
  Stdcompat__init.bytes -> int -> int -> char -> unit = "caml_fill_string"
    "noalloc"

(*
external unsafe_fill :
  Stdcompat__init.bytes -> int -> int -> char -> unit = "fill_string"
    "noalloc"
*)

*)

*)

*)
(** @since 4.08.0:
    external unsafe_fill :
      bytes -> int -> int -> char -> unit = "caml_fill_string"[@@noalloc ]
@since 4.03.0:
external unsafe_fill :
  bytes -> int -> int -> char -> unit = "caml_fill_string"[@@ocaml.deprecated
                                                            "- : bytes -> int -> int -> char -> unit = <fun>"]
[@@noalloc ]
@since 4.02.0:
external unsafe_fill :
  bytes -> int -> int -> char -> unit = "caml_fill_string" "noalloc"[@@ocaml.deprecated
                                                                    "- : bytes -> int -> int -> char -> unit = <fun>"]
 *)

val map : (char -> char) -> string -> string
(** @since 4.00.0: val map : (char -> char) -> string -> string *)

val trim : string -> string
(** @since 4.00.0: val trim : string -> string *)

val iteri : (int -> char -> unit) -> string -> unit
(** @since 4.00.0: val iteri : (int -> char -> unit) -> string -> unit *)

val make : int -> char -> string
(** Alias for {!String.make} *)

external length : string -> int = "%string_length"
(** Alias for {!String.length} *)

external get : string -> int -> char = "%string_safe_get"
(** Alias for {!String.get} *)

val concat : string -> string list -> string
(** Alias for {!String.concat} *)

val compare : t -> t -> int
(** Alias for {!String.compare} *)

val contains_from : string -> int -> char -> bool
(** Alias for {!String.contains_from} *)

val rcontains_from : string -> int -> char -> bool
(** Alias for {!String.rcontains_from} *)

val contains : string -> char -> bool
(** Alias for {!String.contains} *)

val sub : string -> int -> int -> string
(** Alias for {!String.sub} *)

val escaped : string -> string
(** Alias for {!String.escaped} *)

val iter : (char -> unit) -> string -> unit
(** Alias for {!String.iter} *)

val index_from : string -> int -> char -> int
(** Alias for {!String.index_from} *)

val rindex_from : string -> int -> char -> int
(** Alias for {!String.rindex_from} *)

val index : string -> char -> int
(** Alias for {!String.index} *)

val rindex : string -> char -> int
(** Alias for {!String.rindex} *)


val copy : string -> string

(*

val copy : string -> string[@@ocaml.deprecated
                             "- : string -> string = <fun>"]

(*
val copy : string -> string
*)

*)
(** @since 4.08.0: val copy : string -> string
@since 4.02.0:
val copy : string -> string[@@ocaml.deprecated
                             "- : string -> string = <fun>"]
@since 3.07.0: val copy : string -> string
 *)


val uppercase : string -> string

(*

val uppercase : string -> string[@@ocaml.deprecated
                                  "Use String.uppercase_ascii instead."]

(*
val uppercase : string -> string
*)

*)
(** @since 4.08.0: val uppercase : string -> string
@since 4.03.0:
val uppercase : string -> string[@@ocaml.deprecated
                                  "Use String.uppercase_ascii instead."]
@since 3.07.0: val uppercase : string -> string
 *)


val lowercase : string -> string

(*

val lowercase : string -> string[@@ocaml.deprecated
                                  "Use String.lowercase_ascii instead."]

(*
val lowercase : string -> string
*)

*)
(** @since 4.08.0: val lowercase : string -> string
@since 4.03.0:
val lowercase : string -> string[@@ocaml.deprecated
                                  "Use String.lowercase_ascii instead."]
@since 3.07.0: val lowercase : string -> string
 *)


val capitalize : string -> string

(*

val capitalize : string -> string[@@ocaml.deprecated
                                   "Use String.capitalize_ascii instead."]

(*
val capitalize : string -> string
*)

*)
(** @since 4.08.0: val capitalize : string -> string
@since 4.03.0:
val capitalize : string -> string[@@ocaml.deprecated
                                   "Use String.capitalize_ascii instead."]
@since 3.07.0: val capitalize : string -> string
 *)


val uncapitalize : string -> string

(*

val uncapitalize : string -> string[@@ocaml.deprecated
                                     "Use String.uncapitalize_ascii instead."]

(*
val uncapitalize : string -> string
*)

*)
(** @since 4.08.0: val uncapitalize : string -> string
@since 4.03.0:
val uncapitalize : string -> string[@@ocaml.deprecated
                                     "Use String.uncapitalize_ascii instead."]
@since 3.07.0: val uncapitalize : string -> string
 *)

external unsafe_get : string -> int -> char = "%string_unsafe_get"
(** Alias for {!String.unsafe_get} *)

end
